/*
 *   Copyright 2017-2020, Schlumberger
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */

package org.opengroup.osdu.notification.errors;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.exc.UnrecognizedPropertyException;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.exception.ExceptionUtils;
import org.opengroup.osdu.core.common.logging.JaxRsDpsLog;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.Ordered;
import org.springframework.core.annotation.Order;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.context.request.WebRequest;
import org.springframework.web.servlet.mvc.method.annotation.ResponseEntityExceptionHandler;
import org.opengroup.osdu.core.common.model.http.AppException;
import javax.validation.ValidationException;
import java.io.IOException;
import java.nio.file.AccessDeniedException;
import java.util.List;
import java.util.stream.Collectors;

@Order(Ordered.HIGHEST_PRECEDENCE)
@ControllerAdvice
public class SpringExceptionMapper extends ResponseEntityExceptionHandler {

	@Autowired
	private JaxRsDpsLog log;

	@ExceptionHandler(AppException.class)
	protected ResponseEntity<Object> handleAppException(AppException e) {
		return this.getErrorResponse(e);
	}

	@Override
	protected ResponseEntity<Object> handleMethodArgumentNotValid(MethodArgumentNotValidException ex, HttpHeaders headers,
																  HttpStatus status, WebRequest request) {
		List<String> errorList = ex
				.getBindingResult()
				.getFieldErrors()
				.stream()
				.map(fieldError -> fieldError.getField() + " " + fieldError.getDefaultMessage())
				.collect(Collectors.toList());
		return this.getErrorResponse(new AppException(HttpStatus.BAD_REQUEST.value(), "Bad Request", errorList.toString(), ex));
	}

	@ExceptionHandler({ ValidationException.class, JsonProcessingException.class, UnrecognizedPropertyException.class })
	protected ResponseEntity<Object> handleValidationException(Exception e) {
		return this.getErrorResponse(new AppException(HttpStatus.BAD_REQUEST.value(), "Bad request", e.getMessage(), e));
	}

	@ExceptionHandler({ AccessDeniedException.class })
	public ResponseEntity<Object> handleAccessDeniedException(AccessDeniedException ex, WebRequest request) {
		return this.getErrorResponse(new AppException(HttpStatus.UNAUTHORIZED.value(),
				"Unauthorized", ex.getMessage(), ex));
	}

	@ExceptionHandler(Exception.class)
	protected ResponseEntity<Object> handleGeneralException(Exception e) {
		return this.getErrorResponse(
				new AppException(HttpStatus.INTERNAL_SERVER_ERROR.value(), "Server Error",
						"An unknown error has occurred.", e));
	}

	@ExceptionHandler(IOException.class)
	public ResponseEntity<Object> handleIOException(IOException e) {
		if (StringUtils.containsIgnoreCase(ExceptionUtils.getRootCauseMessage(e), "Broken pipe")) {
			this.log.warning("Client closed the connection while request still being processed");
			return null;
		} else {
			return this.getErrorResponse(
					new AppException(HttpStatus.SERVICE_UNAVAILABLE.value(), "Unknown error", e.getMessage(), e));
		}
	}

	private ResponseEntity<Object> getErrorResponse(AppException e) {
		if(e.getOriginalException()!= null) {
			this.log.error(e.getOriginalException().getMessage(), e.getOriginalException());
		}

		if (e.getError().getCode() > 499) {
			this.log.error(e.getError().getMessage(), e);
		} else {
			this.log.warning(e.getError().getMessage(), e);
		}

		return new ResponseEntity<>(e.getError(), HttpStatus.resolve(e.getError().getCode()));
	}
}