/*
 *   Copyright 2017-2020, Schlumberger
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */

package org.opengroup.osdu.notification.auth;

import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.opengroup.osdu.core.common.model.notification.GsaSecret;
import org.opengroup.osdu.core.common.model.notification.GsaSecretValue;
import org.opengroup.osdu.core.common.model.notification.Subscription;
import org.opengroup.osdu.notification.provider.interfaces.IGoogleServiceAccount;
import org.powermock.modules.junit4.PowerMockRunner;
import org.powermock.utils.Asserts;

import java.util.Map;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.when;

@RunWith(PowerMockRunner.class)
public class GsaAuthTest {

    private static final String GOOGLE_ID_TOKEN = "testHeader.testPayload.testSignature";
    private static Subscription gsa_subscription;

    @Mock
    private IGoogleServiceAccount gsaTokenProvider;

    @InjectMocks
    private GsaAuth sut;

    @BeforeClass
    public static void setup() {
        setGsa_subscription();
    }

    private static void setGsa_subscription() {
        gsa_subscription = new Subscription();
        gsa_subscription.setName("gsa_test_subscription");
        gsa_subscription.setPushEndpoint("http:///gsa-challenge");
        gsa_subscription.setDescription("Description");
        gsa_subscription.setTopic("records-changed");
        gsa_subscription.setNotificationId("test-notification-id");
        gsa_subscription.setId("id_1");
        gsa_subscription.setCreatedBy("test@test.com");
        GsaSecret secret = new GsaSecret();
        GsaSecretValue value = new GsaSecretValue();
        value.setAudience("audience");
        value.setKey("{\"keyFile\":{\"key\":\"gsa\"}}");
        secret.setValue(value);
        gsa_subscription.setSecret(secret);
    }

    @Test
    public void should_return_valid_EndpointAndHeaders_gsaClient() throws Exception {
        GsaSecret secret = (GsaSecret) gsa_subscription.getSecret();
        when(this.gsaTokenProvider.getIdToken(any(), any())).thenReturn(GOOGLE_ID_TOKEN);
        sut.setSecret(gsa_subscription.getSecret());
        Asserts.assertNotNull(sut.getSecret(), "Unable to set Secret");
        Map<String, String> headers = sut.getRequestHeaders();
        Assert.assertEquals(GOOGLE_ID_TOKEN, headers.get("Authorization"));
        String pushUrl = sut.getPushUrl(gsa_subscription.getPushEndpoint());
        Assert.assertEquals(pushUrl, gsa_subscription.getPushEndpoint());
    }

    @Test
    public void should_return_emptyHeaders_when_secret_is_null() throws Exception {
        GsaSecret secret = (GsaSecret) gsa_subscription.getSecret();
        when(this.gsaTokenProvider.getIdToken(any(), any())).thenReturn(GOOGLE_ID_TOKEN);
        Map<String, String> headers = sut.getRequestHeaders();
        Assert.assertEquals(0, headers.size());
        String pushUrl = sut.getPushUrl(gsa_subscription.getPushEndpoint());
        Assert.assertEquals(pushUrl, gsa_subscription.getPushEndpoint());
    }

}
