/*
 *   Copyright 2017-2020, Schlumberger
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */

package org.opengroup.osdu.notification.service;

import org.junit.BeforeClass;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.MockitoJUnitRunner;
import org.opengroup.osdu.core.common.http.HttpClient;
import org.opengroup.osdu.core.common.http.HttpResponse;
import org.opengroup.osdu.core.common.logging.JaxRsDpsLog;
import org.opengroup.osdu.core.common.model.notification.GsaSecret;
import org.opengroup.osdu.core.common.model.notification.GsaSecretValue;
import org.opengroup.osdu.core.common.model.notification.HmacSecret;
import org.opengroup.osdu.core.common.model.notification.Subscription;
import org.opengroup.osdu.core.common.notification.SubscriptionException;
import org.opengroup.osdu.notification.auth.factory.AuthFactory;
import org.opengroup.osdu.notification.auth.interfaces.SecretAuth;

import java.util.HashMap;
import java.util.Map;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.fail;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.when;

@RunWith(MockitoJUnitRunner.class)
public class NotificationHandlerTest {

    private static final String NOTIFICATION_ID = "test-notification-id";
    private static final String PUBSUB_MESSAGE = "test-pubsub-message-data";
    private static Subscription gsa_subscription;
    private static Subscription hmac_subscription;
    private final HttpResponse response = new HttpResponse();

    @Mock
    private SubscriptionHandler subscriptionHandler;

    @Mock
    private AuthFactory authFactory;

    @Mock
    private SecretAuth secretAuth;

    @Mock
    private HttpClient httpClient;

    @Mock
    private JaxRsDpsLog log;

    @InjectMocks
    private NotificationHandler sut;

    @BeforeClass
    public static void setup() {
        setHmac_subscription();
        setGsa_subscription();
    }

    private static void setGsa_subscription() {
        gsa_subscription = new Subscription();
        gsa_subscription.setName("gsa_test_subscription");
        gsa_subscription.setPushEndpoint("http:///gsa-challenge");
        gsa_subscription.setDescription("Description");
        gsa_subscription.setTopic("records-changed");
        gsa_subscription.setNotificationId(NOTIFICATION_ID);
        gsa_subscription.setId("id_1");
        gsa_subscription.setCreatedBy("test@test.com");
        GsaSecret secret = new GsaSecret();
        GsaSecretValue value = new GsaSecretValue();
        value.setAudience("audience");
        value.setKey("keyFile");
        secret.setValue(value);
        gsa_subscription.setSecret(secret);
    }

    private static void setHmac_subscription() {
        hmac_subscription = new Subscription();
        hmac_subscription.setName("hamc_test_subscription");
        hmac_subscription.setPushEndpoint("http://challenge");
        hmac_subscription.setDescription("Description");
        hmac_subscription.setTopic("records-changed");
        hmac_subscription.setNotificationId(NOTIFICATION_ID);
        hmac_subscription.setId("id_1");
        hmac_subscription.setCreatedBy("test@test.com");
        HmacSecret secret = new HmacSecret();
        secret.setValue("testsecret");
        hmac_subscription.setSecret(secret);
    }

    @Test
    public void should_return200_whenPubsubMessageValidAndSuccessCodeReturnedFromClient_gsa() throws Exception {
        response.setResponseCode(200);
        Map<String, String> headers = new HashMap<String, String>();
        when(this.subscriptionHandler.getSubscriptionFromCache(NOTIFICATION_ID)).thenReturn(gsa_subscription);
        when(this.authFactory.getSecretAuth(any())).thenReturn(secretAuth);
        when(this.httpClient.send(any())).thenReturn(response);
        when(this.secretAuth.getPushUrl(gsa_subscription.getPushEndpoint())).thenReturn(gsa_subscription.getPushEndpoint());
        when(this.secretAuth.getRequestHeaders()).thenReturn(headers);
        HttpResponse response = this.sut.notifySubscriber(NOTIFICATION_ID, PUBSUB_MESSAGE, headers);
        assertEquals(200, response.getResponseCode());
    }

    @Test
    public void should_return200_whenPubsubMessageValidAndSuccessCodeReturnedFromClient_hmac() throws Exception {
        response.setResponseCode(200);
        Map<String, String> headers = new HashMap<String, String>();
        when(this.subscriptionHandler.getSubscriptionFromCache(NOTIFICATION_ID)).thenReturn(hmac_subscription);
        when(this.authFactory.getSecretAuth(any())).thenReturn(secretAuth);
        when(this.httpClient.send(any())).thenReturn(response);
        when(this.secretAuth.getPushUrl(hmac_subscription.getPushEndpoint())).thenReturn(hmac_subscription.getPushEndpoint());
        when(this.secretAuth.getRequestHeaders()).thenReturn(headers);
        HttpResponse response = this.sut.notifySubscriber(NOTIFICATION_ID, PUBSUB_MESSAGE, headers);
        assertEquals(200, response.getResponseCode());
    }

    @Test(expected = SubscriptionException.class)
    public void should_throwException_whenSubscriptionHandlerThrowsException() throws Exception {
        Map<String, String> headers = new HashMap<String, String>();
        when(subscriptionHandler.getSubscriptionFromCache(NOTIFICATION_ID)).thenThrow(new SubscriptionException("error", response));
        this.sut.notifySubscriber(NOTIFICATION_ID, PUBSUB_MESSAGE, headers);
        fail("should throw SubscriptionException");
    }
}
